/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package gaiacrtn.config;

import gaiacrtn.swing.CommandHolder;
import gaiacrtn.swing.MenuDefFactory;
import gaiacrtn.config.PropertiesApp;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

/** Use this class to load a resource bundle from a property file.
 *
 */
public class ResourceApp extends PropertyResourceBundle {
   /** Application resource bundle.
    * Initialized by setAppRes().
    */
   static public ResourceBundle res;

   /** Application menu defition factory.
    * Initialized by setAppMenuDefFactory();
    */
   static public MenuDefFactory factoryMenuDef;

   /** Application command holder.
    * Initialized by setAppCommandHoler();
    */
   static public CommandHolder cmdHolder;

   /** Application home directory file.
    */
   static private File _fileAppHome;

   /** Set the resource bundle for the application.
    * @param res Resource bundle for the application.
    * @see gaiacrtn.config.ResourceApp.res
    */
   static public void setResApp( ResourceBundle resApp ) {
      res = resApp;
   }

   /** Sets the menu definition factory used by the application.
    * @param factoryMenu Menu factory.
    * @see factoryMenu.
    */
   static public void setAppMenuDefFactory( MenuDefFactory factory ) {
      factoryMenuDef = factory;
   }

   /** Sets the command holder used by the application.
    * @param commandHolder Command holder.
    */
   static public void setAppCommandHolder( CommandHolder commandHolder ) {
      cmdHolder = commandHolder;
   }

   /** Gets a string from the application resource bundle.
    * @param strKey Key of the string resource to retreive.
    * @return String for that resource.
    */
   static public String getResString( String strKey )
         throws MissingResourceException {
      return res.getString( strKey );
   }

   /** Creates a resource bundle based on <code>Properties</code> from
    * an input stream.
    */
   protected ResourceApp( InputStream is ) throws IOException {
      super( is );
   }

   /** Gets an absolute file from a file that is relative to the application
    * Home directory.
    * @param fileRelative File that is relative to the application
    *                           directory.
    * @return An absolute file that point to the given file.
    */
   static public File appFile( String strFilenameRelative ) {
      if ( _fileAppHome == null )
         _fileAppHome = new File( PropertiesApp.appHome() );
      return new File( _fileAppHome, strFilenameRelative );
   }

   static public String appFilename( String strFilenameRelative ) {
      return PropertiesApp.appHome() + File.separator + strFilenameRelative;
   }

   /** Create a resource bundle for the application.
    * Get the resources bundle for the default locale in the home application
    * directory in the folder "resources" with the application name as a
    * base. For example if the application home is "g:\app" and the application
    * name is "myApp" then the base of the filename for the resource file is
    * "g:\app\resources\myApp".
    * <p>
    * The application home directory and name are retreive using
    * <code>PropertiesApp</code>.
    */
   static public ResourceApp create() {
      return create( appFilename( "resources" +
                                  File.separator + PropertiesApp.appName() ),
                     Locale.getDefault() );
   }

   /** Creates a resource bundle based on a property file.
    * The filename rule for the property file is the same as for the resource
    * bundle:
    * if <code>strFilename</code> is <I>"myApp"</I> and the locale is
    * <I>"fr_FR"</I> and the default locale is <I>en_US</I> then the file
    * that be searched are the following:
    * <BLOCKQUOTE><PRE>
    * myApp_fr_FR.properties
    * myApp_fr.properties
    * myApp_en_US.properties
    * myApp_en.properties
    * myApp.properties
    * </PRE></BLOCKQUOTE>
    * @param strFilename Base of the filename for the resource bundle.
    * @param locale Locale for this properties.
    */
   static public ResourceApp create( String strFilename,
                                     Locale locale )
         throws MissingResourceException {
      // find resource filename
      ResourceFilenameFinder finder = new ResourceFilenameFinder();
      if ( !finder.findFilename( strFilename, locale ) ) {
         throw new MissingResourceException(
               "Failed to load resource bundle for the application:\n" +
               "Filename = " + strFilename + ".properties", null, null );
      }

      // creates the ResourceApp
      try {
         ResourceApp resApp = new ResourceApp(
               new FileInputStream( finder._strFilename ) );
         return resApp;
      } catch (IOException e) {
         throw new MissingResourceException(
               "Failed to read the application resource file.\n" +
               "Filename = " + finder._strFilename + "\n" + e, null, null );
      }
   }
}

/** This class helps locating the resource filename depending on the locale.
 *
 */
class ResourceFilenameFinder {
   public String _strFilename;
   public Locale _locale;

   boolean findFilename( String strFilename, Locale locale ) {
      if ( !checkResourceAvailable( strFilename, locale ) )
         if ( !checkResourceAvailable( strFilename, Locale.getDefault() ) )
            return checkResourceAvailable( strFilename, null );
      return true;
   }

   protected boolean checkResourceAvailable( String strFilename,
                                             Locale locale ) {
      int nStageMax = 1;
      if ( locale != null ) {
         nStageMax = 2;
         strFilename += "_";
         strFilename += locale.getCountry();
      }

      for (int nStage = 0; nStage < nStageMax; ++nStage) {
         String strFile = strFilename;
         if ( nStage == 0 && locale != null )
            strFile += "_" + locale.getVariant();
         strFile += ".properties";

         if ( new File( strFile ).exists() ) {
            _strFilename = strFile;
            _locale = locale;
            return true;
         }
      }

      return false;
   }
}
