/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */

/* Change:
 * 2000/05/10: - If a command is not registered, then it is automatically
 *               created with a null ActionListener.
 */
package gaiacrtn.swing;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

/** This class holds a list of commands and their associated action.
 * Commands are registered into the framework using addCommand(). When a
 * command is registered, the framework retreive the resource related to
 * this command and creates two Action objects : one for the command, and
 * one to use in menu (which usually have longer label).
 * <P>
 * Here is the way resource are looked up:<P>
 * <B>NAME</B> is the command name ( "quit" for example ).
 * <UL>
 * <LI>cmd.<B>NAME</B>_label : label for the command when use in button
 *     (short desc.). This resource is mandatory.</LI>
 * <LI>cmd.<B>NAME</B>_menu : label for the command when use in menu item
 *     (long desc.). This resource is optionnal.</LI>
 * <LI>cmd.<B>NAME</B>_icon : icon file path relative to the application home
 *     directory. For example "resources/new.gif". This resource is optionnal.
 *     </LI>
 * </UL>
 * <P>
 * For example, if the command name is "quit", you need to add the following
 * resources in the resource properties file:<P>
 * <PRE>
 * cmd.quit_label="Quit"
 * cmd.quit_menu="Quit"
 * cmd.quit_icon="resources/quit.gif"
 * </PRE>
 */
public class CommandHolder {
   private Map _mapAction = new HashMap();
   private Map _mapActionMenu = new HashMap();

   public CommandHolder() {
   }

   /** Setup a button using the description of the command it implements.
    * @param button Button to setup. Set its label, icon and action listener.
    * @param strCommandName Name of the command the button implements.
    */
   public void setup( AbstractButton button, String strCommandName ) {
      Action action = action( strCommandName );
      button.setText( (String) action.getValue( Action.NAME ) );
      button.setIcon( (Icon) action.getValue( Action.SMALL_ICON ) );
      button.addActionListener( action );
   }

   /** Get the action for a given command.
    * @param strCommandName Name of the command.
    * @return Action with text, icon and description. null if no command
    *         with the correct name is found.
    */
   public Action action( String strCommandName ) {
      return retreiveAction( _mapAction, strCommandName );
   }

   /** Get the action for a menu for a given command.
    * @param strCommandName Name of the command.
    * @return Action with text, icon and description. null if no command
    *         with the correct name is found.
    */
   public Action actionMenu( String strCommandName ) {
      return retreiveAction( _mapActionMenu, strCommandName );
   }

   /** Retreives an action for a given command.
    * @param mapAction Map that contains the registered action.
    * @param strCommandName Name of the command.
    * @return Action. If the command was not registered, it created and
    *         a \c null ActionListener is provided.
    */
   protected Action retreiveAction( Map mapAction, String strCommandName ) {
      Action action = (Action) mapAction.get( strCommandName );
      if ( action == null )   // action not registered !!!
      {
         System.err.println( "Command not registered: " + strCommandName );
         addCommand( strCommandName, null );
         action = (Action) mapAction.get( strCommandName );
      }
      return action;
   }

   /** Register a new command.
    * @param strCommandName Name of the command.
    * @param actionListener Object that will perform the action.
    */
   public void addCommand( String strCommandName,
                           ActionListener actionListener ) {
      CommandInfo cmdInfo = new CommandInfo( strCommandName );
      _mapAction.put( strCommandName,
                      makeAction( cmdInfo, actionListener, false ) );
      _mapActionMenu.put( strCommandName,
                          makeAction( cmdInfo, actionListener, true ) );
   }

   protected Action makeAction( CommandInfo cmdInfo,
                                ActionListener actionListener,
                                boolean bIsMenu ) {
      String strLabel = ( bIsMenu && cmdInfo.strLabelMenu != null ) ?
            cmdInfo.strLabelMenu :
            cmdInfo.strLabel;
      Action action = new ActionWrapper( strLabel, actionListener );
      if ( cmdInfo.iconCmd != null )
         action.putValue( Action.SMALL_ICON, cmdInfo.iconCmd );
      if ( actionListener == null )
         action.setEnabled( false );
      return action;
   }

   // internal class

   /** This class redirect the action performing to an ActionListener.
    */
   protected class ActionWrapper extends AbstractAction {
      private ActionListener _actionListener;

      /** Constructs the action wrapper.
       * @param strCommandName Name of the command. Used to retreive
       *                       the action label, icons, description...
       * @param actionListener ActionListener which is used when the
       *                       action is performed.
       */
      public ActionWrapper( String strCommandName,
                            ActionListener actionListener ) {
         super( strCommandName );
         _actionListener = actionListener;
      }

      public void actionPerformed( ActionEvent e ) {
         if ( _actionListener != null )
            _actionListener.actionPerformed( e );
      }
   }
}