package gaiacrtn.util;

/** This interface indicates the capability of an object to be switched (visited).
 * The {@link gaiacrtn.util.Switchable} and {@link gaiacrtn.util.Switch} interfaces
 * are extensions of the design patterns known as "Visitor".
 * <p>
 * Here is an example of how an object should implements the
 * {@link #apply apply() }  method.
 * <p>
 * <pre>
 * class StatementIf implements Switchable {
 *   public void apply( Switch switch ) {
 *     ((SwitchStatement)sw).caseStatementIf( this );
 *   }
 * }
 * </pre>
 * You can see that you must cast the <code>switch</code> to its proper type,
 * and then call the method for this type.
 * <p>
 * See {@link Switch} for more explanation about the use
 * of this pattern.
 * @see Switchable
 */
public interface Switchable {
   /** Method called to switch an object.
    * The implementation of this method should call a method on the <code>
    * sw</code> after casting to the correct {@link Switch} type. The method
    * that is called should depend on the type of the object. Naming convention
    * for the called method name is "case" followed by the type of the object.
    * @param sw Switch on which the case method should be called.
    */
   public void apply( Switch sw );
}